"""
The :mod:`usklearn.datasets` module includes utilities to load
popular uplift reference datasets. It also features some artificial
data generators.
"""

from .Hillstrom import fetch_Hillstrom
from .Lalonde import fetch_Lalonde
from .TelecomChurn import fetch_TelecomChurn
from .Criteo import fetch_Criteo
from .Lenta import fetch_Lenta
from .colon import fetch_colon
from .Starbucks import fetch_Starbucks
from .Megafon import fetch_Megafon
from .Lazada import fetch_Lazada
from .Information import fetch_Information
from .MarketingAB import fetch_marketing_AB
